// -*- mode: delphi; -*-

{
 Basic example of LADSPA Plug-in written in Pascal,
 for more information: http://www.ladspa.org
 
 I tested it on Windows with FreePascal 2.0.2 and it loads/works well with
 Audacity 1.2.4 (put the DLL file in Audacity's Plug-ins directory)
 
 To compile: fpc -Mobjfpc amp.pas
 
 Copyright (c) 2006 Javier Valcarce Garca, <javier.valcarce@gmail.com>    
                                                                           
 Permission to use, copy, modify, and distribute this software and its     
 documentation under the terms of the GNU General Public License is hereby 
 granted. No representations are made about the suitability of this        
 software for any purpose. It is provided "as is" without expressed or     
 implied warranty. See the GNU General Public License for more details.    

 $Id: amp.dpr 22 2006-08-22 15:25:05Z javg $
}

library amp;
uses
  math,
  ladspa in 'ladspa.pas';

const
  NPORTS = 3; // Number of ports
  PORT_I = 0; // Input
  PORT_O = 1; // Output
  PORT_C = 2; // Control (gain in this case)

type
  PCharArray                  = packed array [0..NPORTS-1] of PChar;
  LADSPA_PortDescriptorArray  = packed array [0..NPORTS-1] of LADSPA_PortDescriptor;
  LADSPA_PortRangeHintArray   = packed array [0..NPORTS-1] of LADSPA_PortRangeHint;

  PPCharArray                 = ^PCharArray;
  PLADSPA_PortDescriptorArray = ^LADSPA_PortDescriptorArray;
  PLADSPA_PortRangeHintArray  = ^LADSPA_PortRangeHintArray;

var
  d           : PLADSPA_Descriptor;
  portn       : PPCharArray;
  portd       : PLADSPA_PortDescriptorArray;
  portr       : PLADSPA_PortRangeHintArray;


type
  PTPluginData = ^TPluginData;
  TPluginData  =
  packed record
    fs      : LADSPA_Data;

    // Ports...
    port_i : PLADSPA_Data;
    port_o : PLADSPA_Data;
    port_c : PLADSPA_Data;
  end;


///////////////////////////////////////////////////////////////////////////////////////////////////
function ladspa_descriptor(index : LongWord): PLADSPA_Descriptor; cdecl; export;
begin

  case index of
     0: ladspa_descriptor := d;
  else  ladspa_descriptor := nil;
  end;

end;


///////////////////////////////////////////////////////////////////////////////////////////////////
function MyInstantiate(Descriptor : PLADSPA_Descriptor; SampleRate : LongWord) : LADSPA_Handle; cdecl;
var
   p : PTPluginData;
begin
   new(p);
   p^.fs  := SampleRate;
   // ...

  Result := LADSPA_Handle(p);
end;


///////////////////////////////////////////////////////////////////////////////////////////////////
procedure MyConnectPort(Instance : LADSPA_Handle; Port : LongWord; DataLocation : PLADSPA_Data); cdecl;
var
  p : PTPluginData;
begin
  p := PTPluginData(Instance);

  case Port of
    PORT_I: p^.port_i := DataLocation;
    PORT_O: p^.port_o := DataLocation;
    PORT_C: p^.port_c := DataLocation;
  end;
end;




///////////////////////////////////////////////////////////////////////////////////////////////////
procedure MyRun(Instance : LADSPA_Handle; SampleCount : LongWord); cdecl;
type
   LADSPA_DataArray = array of LADSPA_Data;
var
  p  : PTPluginData;
  pi : LADSPA_DataArray;
  po : LADSPA_DataArray;
  p1 : LADSPA_Data;
  i  : LongWord;

begin

   p  := PTPluginData(Instance);

   pi := LADSPA_DataArray(p^.port_i);  // input port
   po := LADSPA_DataArray(p^.port_o);  // output port
   p1 := p^.port_c^;                   // gain
   
   for i := 0 to SampleCount - 1 do
   begin
         po[i] := pi[i] * p1;   
   end;
end;



///////////////////////////////////////////////////////////////////////////////////////////////////
procedure MyCleanup(Instance : LADSPA_Handle); cdecl;
var p : PTPluginData;
begin
  p := PTPluginData(Instance);

  dispose(p)
end;


///////////////////////////////////////////////////////////////////////////////////////////////////
exports ladspa_descriptor;
///////////////////////////////////////////////////////////////////////////////////////////////////


initialization
   // This code executes after the DLL has been loaded
     
   new(d);

   with d^ do
   begin
      UniqueID        := 1800; // Must be unique among other plugins!
      
      Properties      := LADSPA_PROPERTY_HARD_RT_CAPABLE;
      Label0          := 'amp';
      Name            := 'Amplifier (example of LADSPA plug-in in Pascal)';
      Maker           := 'Javier Valcarce Garca, <javier.valcarce@gmail.com>';
      Copyright       := 'GPL';
      PortCount       := NPORTS;
      
      new(portn);
      new(portd);
      new(portr);
      
      PortNames       := @(portn^[0]);
      PortDescriptors := @(portd^[0]);
      PortRangeHints  := @(portr^[0]);
      
      portn^[PORT_I]                := 'Input';
      portd^[PORT_I]                := LADSPA_PORT_INPUT  or LADSPA_PORT_AUDIO;
      portr^[PORT_I].HintDescriptor := 0;
      
      portn^[PORT_O]                := 'Output';
      portd^[PORT_O]                := LADSPA_PORT_OUTPUT or LADSPA_PORT_AUDIO;
      portr^[PORT_O].HintDescriptor := 0;
      
      portn^[PORT_C]                := 'Gain';
      portd^[PORT_C]                := LADSPA_PORT_INPUT  or LADSPA_PORT_CONTROL;
      portr^[PORT_C].HintDescriptor := LADSPA_HINT_BOUNDED_BELOW or LADSPA_HINT_BOUNDED_ABOVE;
      
      portr^[PORT_C].LowerBound     := 0;
      portr^[PORT_C].UpperBound     := 2;
    
      // Callbacks called from HOST      
      instantiate         := @MyInstantiate;  
      connect_port        := @MyConnectPort;  
      activate            := nil;
      run                 := @MyRun;          
      run_adding          := nil;            
      set_run_adding_gain := nil;            
      deactivate          := nil;            
      cleanup             := @MyCleanup;      
  end;
   
   
finalization
   
   dispose(portn);
   dispose(portd);
   dispose(portr); 
   dispose(d);
end.
